@@ -4,7 +4,7 @@
  * This file is part of the Predis package.
  *
  * (c) 2009-2020 Daniele Alessandri
- * (c) 2021-2024 Till Krüss
+ * (c) 2021-2025 Till Krüss
  *
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
@@ -20,15 +20,22 @@ use Predis\ClientException;
 use Predis\Cluster\RedisStrategy as RedisClusterStrategy;
 use Predis\Cluster\SlotMap;
 use Predis\Cluster\StrategyInterface;
+use Predis\Command\Command;
 use Predis\Command\CommandInterface;
 use Predis\Command\RawCommand;
+use Predis\Connection\AbstractAggregateConnection;
 use Predis\Connection\ConnectionException;
 use Predis\Connection\FactoryInterface;
 use Predis\Connection\NodeConnectionInterface;
+use Predis\Connection\ParametersInterface;
+use Predis\Connection\RelayFactory;
 use Predis\NotSupportedException;
 use Predis\Response\Error as ErrorResponse;
 use Predis\Response\ErrorInterface as ErrorResponseInterface;
 use Predis\Response\ServerException;
+use Predis\Retry\Retry;
+use Predis\Retry\Strategy\ExponentialBackoff;
+use Predis\TimeoutException;
 use ReturnTypeWillChange;
 use Throwable;
 use Traversable;
@@ -53,9 +60,13 @@ use Traversable;
  * Asking for the cluster configuration to Redis is actually done by issuing a
  * CLUSTER SLOTS command to a random node in the pool.
  */
-class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
+class RedisCluster extends AbstractAggregateConnection implements ClusterInterface, IteratorAggregate, Countable
 {
-    private $useClusterSlots = true;
+    public $useClusterSlots = true;
+
+    /**
+     * @var NodeConnectionInterface[]
+     */
     private $pool = [];
     private $slots = [];
     private $slotmap;
@@ -64,17 +75,35 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
     private $retryLimit = 5;
     private $retryInterval = 10;
 
+    /**
+     * @var int
+     */
+    private $readTimeout = 1000;
+
+    /**
+     * @var ParametersInterface
+     */
+    private $connectionParameters;
+
     /**
      * @param FactoryInterface       $connections Optional connection factory.
      * @param StrategyInterface|null $strategy    Optional cluster strategy.
+     * @param int|null               $readTimeout Optional read timeout
      */
     public function __construct(
         FactoryInterface $connections,
-        ?StrategyInterface $strategy = null
+        ParametersInterface $parameters,
+        ?StrategyInterface $strategy = null,
+        ?int $readTimeout = null
     ) {
         $this->connections = $connections;
+        $this->connectionParameters = $parameters;
         $this->strategy = $strategy ?: new RedisClusterStrategy();
         $this->slotmap = new SlotMap();
+
+        if (!is_null($readTimeout)) {
+            $this->readTimeout = $readTimeout;
+        }
     }
 
     /**
@@ -130,7 +159,7 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
      */
     public function connect()
     {
-        if ($connection = $this->getRandomConnection()) {
+        foreach ($this->pool as $connection) {
             $connection->connect();
         }
     }
@@ -235,35 +264,31 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
      */
     private function queryClusterNodeForSlotMap(NodeConnectionInterface $connection)
     {
-        $retries = 0;
-        $retryAfter = $this->retryInterval;
+        // Backward-compatible hardcoded retry
+        $retry = new Retry(
+            new ExponentialBackoff($this->retryInterval * 1000, -1),
+            $this->retryLimit,
+            [ConnectionException::class]
+        );
+
         $command = RawCommand::create('CLUSTER', 'SLOTS');
 
-        while ($retries <= $this->retryLimit) {
-            try {
-                $response = $connection->executeCommand($command);
-                break;
-            } catch (ConnectionException $exception) {
-                $connection = $exception->getConnection();
-                $connection->disconnect();
+        $doCallback = function () use (&$connection, $command) {
+            return $connection->executeCommand($command);
+        };
 
-                $this->remove($connection);
+        $failCallback = function (ConnectionException $exception) use (&$connection) {
+            $connection = $exception->getConnection();
+            $connection->disconnect();
 
-                if ($retries === $this->retryLimit) {
-                    throw $exception;
-                }
+            $this->remove($connection);
 
-                if (!$connection = $this->getRandomConnection()) {
-                    throw new ClientException('No connections left in the pool for `CLUSTER SLOTS`');
-                }
-
-                usleep($retryAfter * 1000);
-                $retryAfter *= 2;
-                ++$retries;
+            if (!$connection = $this->getRandomConnection()) {
+                throw new ClientException('No connections left in the pool for `CLUSTER SLOTS`');
             }
-        }
+        };
 
-        return $response;
+        return $retry->callWithRetry($doCallback, $failCallback);
     }
 
     /**
@@ -358,9 +383,9 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
 
         if (isset($this->slots[$slot])) {
             return $this->slots[$slot];
-        } else {
-            return $this->getConnectionBySlot($slot);
         }
+
+        return $this->getConnectionBySlot($slot);
     }
 
     /**
@@ -520,51 +545,42 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
      * @param string           $method  Actual method.
      *
      * @return mixed
+     * @throws Throwable
      */
     private function retryCommandOnFailure(CommandInterface $command, $method)
     {
-        $retries = 0;
-        $retryAfter = $this->retryInterval;
-
-        while ($retries <= $this->retryLimit) {
-            try {
-                $response = $this->getConnectionByCommand($command)->$method($command);
-
-                if ($response instanceof ErrorResponse) {
-                    $message = $response->getMessage();
-
-                    if (strpos($message, 'CLUSTERDOWN') !== false) {
-                        throw new ServerException($message);
-                    }
-                }
-
-                break;
-            } catch (Throwable $exception) {
-                usleep($retryAfter * 1000);
-                $retryAfter *= 2;
-
-                if ($exception instanceof ConnectionException) {
-                    $connection = $exception->getConnection();
-
-                    if ($connection) {
-                        $connection->disconnect();
-                        $this->remove($connection);
-                    }
-                }
-
-                if ($retries === $this->retryLimit) {
-                    throw $exception;
-                }
-
-                if ($this->useClusterSlots) {
-                    $this->askSlotMap();
-                }
-
-                ++$retries;
-            }
+        if ($this->connectionParameters->isDisabledRetry() || $this->connections instanceof RelayFactory) {
+            // Override default parameters, for backward-compatibility
+            // with current behaviour
+            $retry = new Retry(
+                new ExponentialBackoff($this->retryInterval * 1000, -1),
+                $this->retryLimit
+            );
+        } else {
+            $retry = $this->connectionParameters->retry;
         }
+        $retry->updateCatchableExceptions([ServerException::class]);
 
-        return $response;
+        $doCallback = function () use ($command, $method) {
+            $response = $this->getConnectionByCommand($command)->$method($command);
+
+            if ($response instanceof ErrorResponse) {
+                $message = $response->getMessage();
+
+                if (strpos($message, 'CLUSTERDOWN') !== false) {
+                    throw new ServerException($message);
+                }
+            }
+
+            return $response;
+        };
+
+        return $retry->callWithRetry(
+            $doCallback,
+            function (Throwable $e) {
+                $this->onFailCallback($e);
+            }
+        );
     }
 
     /**
@@ -597,6 +613,20 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
         return $response;
     }
 
+    /**
+     * {@inheritdoc}
+     */
+    public function executeCommandOnEachNode(CommandInterface $command): array
+    {
+        $responses = [];
+
+        foreach ($this->pool as $connection) {
+            $responses[] = $connection->executeCommand($command);
+        }
+
+        return $responses;
+    }
+
     /**
      * @return int
      */
@@ -640,12 +670,9 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
     }
 
     /**
-     * Returns the underlying command hash strategy used to hash commands by
-     * using keys found in their arguments.
-     *
-     * @return StrategyInterface
+     * {@inheritDoc}
      */
-    public function getClusterStrategy()
+    public function getClusterStrategy(): StrategyInterface
     {
         return $this->strategy;
     }
@@ -678,4 +705,60 @@ class RedisCluster implements ClusterInterface, IteratorAggregate, Countable
     {
         $this->useClusterSlots = (bool) $value;
     }
+
+    /**
+     * {@inheritdoc}
+     */
+    public function getParameters(): ?ParametersInterface
+    {
+        return $this->connectionParameters;
+    }
+
+    /**
+     * Loop over connections until there's data to read.
+     *
+     * @return mixed
+     */
+    public function read()
+    {
+        while (true) {
+            foreach ($this->pool as $connection) {
+                if ($connection->hasDataToRead()) {
+                    return $connection->read();
+                }
+            }
+
+            usleep($this->readTimeout);
+        }
+    }
+
+    /**
+     * Handle exceptions.
+     *
+     * @param  Throwable $exception
+     * @return void
+     */
+    private function onFailCallback(Throwable $exception)
+    {
+        if ($exception instanceof ConnectionException) {
+            $connection = $exception->getConnection();
+
+            if ($connection) {
+                $connection->disconnect();
+                $this->remove($connection);
+            }
+
+            if ($this->useClusterSlots) {
+                $this->askSlotMap();
+            }
+        }
+
+        if ($exception instanceof TimeoutException) {
+            $connection = $exception->getConnection();
+
+            if ($connection) {
+                $connection->disconnect();
+            }
+        }
+    }
 }
