@@ -508,6 +508,9 @@ class PredisStrategyTest extends PredisTestCase
 
             /* cluster */
             'CLUSTER' => 'keys-fake',
+
+            /* control */
+            'ACL' => 'keys-fake',
         ];
 
         if (isset($type)) {
