@@ -50,6 +50,20 @@ class LPUSHX_Test extends PredisCommandTestCase
         $this->assertSame($expected, $command->getArguments());
     }
 
+    /**
+     * @group disconnected
+     */
+    public function testFilterArgumentsValuesAsSingleArray(): void
+    {
+        $arguments = ['key', ['value1', 'value2', 'value3']];
+        $expected = ['key', 'value1', 'value2', 'value3'];
+
+        $command = $this->getCommand();
+        $command->setArguments($arguments);
+
+        $this->assertSame($expected, $command->getArguments());
+    }
+
     /**
      * @group disconnected
      */
