@@ -12,6 +12,7 @@
 
 namespace Predis\Command\Redis;
 
+use Predis\Command\PrefixableCommand;
 use Predis\Response\ServerException;
 
 class ZRANDMEMBER_test extends PredisCommandTestCase
@@ -93,6 +94,23 @@ class ZRANDMEMBER_test extends PredisCommandTestCase
         $this->assertNull($redis->zrandmember($notExpectedKey));
     }
 
+    /**
+     * @group disconnected
+     */
+    public function testPrefixKeys(): void
+    {
+        /** @var PrefixableCommand $command */
+        $command = $this->getCommand();
+        $actualArguments = ['key', 10];
+        $prefix = 'prefix:';
+        $expectedArguments = ['prefix:key', 10];
+
+        $command->setArguments($actualArguments);
+        $command->prefixKeys($prefix);
+
+        $this->assertSame($expectedArguments, $command->getArguments());
+    }
+
     /**
      * @group connected
      * @requiresRedisVersion >= 6.2.0
