@@ -19,8 +19,14 @@ use Predis\Command;
 use Predis\Connection;
 use Predis\Connection\FactoryInterface;
 use Predis\Connection\Parameters;
+use Predis\Connection\Resource\StreamFactoryInterface;
+use Predis\Connection\StreamConnection;
 use Predis\Response;
+use Predis\Retry\Retry;
+use Predis\Retry\Strategy\ExponentialBackoff;
 use PredisTestCase;
+use Psr\Http\Message\StreamInterface;
+use RuntimeException;
 
 class RedisClusterTest extends PredisTestCase
 {
@@ -1373,6 +1379,59 @@ class RedisClusterTest extends PredisTestCase
         $cluster->executeCommand($command);
     }
 
+    /**
+     * @medium
+     * @group disconnected
+     * @group slow
+     */
+    public function testRetryCommandFailureOnCustomRetryConfiguration()
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+        ]);
+
+        $mockStream
+            ->expects($this->exactly(4))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(4))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                1000,
+                1000
+            );
+
+        $mockStream
+            ->expects($this->once())
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+OK\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $connection = new StreamConnection($parameters, $mockStreamFactory);
+        $cluster = new RedisCluster(new Connection\Factory(), $parameters);
+        $cluster->useClusterSlots(false);
+        $cluster->add($connection);
+
+        $this->assertEquals(
+            'OK',
+            $cluster->executeCommand(Command\RawCommand::create('SET', 1001))
+        );
+    }
+
     /**
      * @medium
      * @group disconnected
