@@ -20,9 +20,20 @@ use Predis\ClientInterface;
 use Predis\Command\CommandInterface;
 use Predis\Command\Redis\ECHO_;
 use Predis\Command\Redis\PING;
+use Predis\Connection\Cluster\RedisCluster;
+use Predis\Connection\Factory;
 use Predis\Connection\Parameters;
+use Predis\Connection\Replication\MasterSlaveReplication;
+use Predis\Connection\Resource\StreamFactoryInterface;
+use Predis\Connection\StreamConnection;
+use Predis\Replication\ReplicationStrategy;
 use Predis\Response;
+use Predis\Retry\Retry;
+use Predis\Retry\Strategy\ExponentialBackoff;
+use Predis\TimeoutException;
 use PredisTestCase;
+use Psr\Http\Message\StreamInterface;
+use RuntimeException;
 use stdClass;
 
 class PipelineTest extends PredisTestCase
@@ -91,7 +102,7 @@ class PipelineTest extends PredisTestCase
             ->willReturn($object);
 
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(2))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -119,7 +130,7 @@ class PipelineTest extends PredisTestCase
             ->willReturn($error);
 
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(2))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -145,7 +156,7 @@ class PipelineTest extends PredisTestCase
             ->willReturn($error);
 
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(2))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -238,7 +249,7 @@ class PipelineTest extends PredisTestCase
             ->willReturnCallback($this->getReadCallback());
 
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(2))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -301,7 +312,7 @@ class PipelineTest extends PredisTestCase
             ->willReturnCallback($this->getReadCallback());
 
         $connection
-            ->expects($this->exactly(2))
+            ->expects($this->exactly(4))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -333,15 +344,15 @@ class PipelineTest extends PredisTestCase
             ->expects($this->once())
             ->method('switchToMaster');
         $connection
-            ->expects($this->exactly(3))
+            ->expects($this->exactly(6))
             ->method('getConnectionByCommand')
             ->willReturn($nodeConnection);
-        $connection
+        $nodeConnection
             ->expects($this->exactly(3))
             ->method('readResponse')
             ->willReturn($pong);
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(3))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -429,7 +440,7 @@ class PipelineTest extends PredisTestCase
             ->method('readResponse')
             ->willReturnCallback($this->getReadCallback());
         $connection
-            ->expects($this->once())
+            ->expects($this->exactly(2))
             ->method('getParameters')
             ->willReturn(new Parameters(['protocol' => 2]));
 
@@ -478,6 +489,327 @@ class PipelineTest extends PredisTestCase
         $this->assertNull($responses);
     }
 
+    /**
+     * @group disconnected
+     * @throws Exception
+     */
+    public function testRetryStandalonePipelineOnRetryableErrors(): void
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+        ]);
+
+        $mockStream
+            ->expects($this->atLeast(3))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(4))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                1000
+            );
+
+        $mockStream
+            ->expects($this->exactly(3))
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+PONG\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $connection = new StreamConnection($parameters, $mockStreamFactory);
+        $pipeline = new Pipeline(new Client($connection));
+
+        $responses = $pipeline->execute(function (Pipeline $pipe) {
+            $pipe->ping();
+            $pipe->ping();
+            $pipe->ping();
+        });
+
+        $this->assertEquals(['PONG', 'PONG', 'PONG'], $responses);
+    }
+
+    /**
+     * @group disconnected
+     * @throws Exception
+     */
+    public function testRetryClusterPipelineOnRetryableErrors(): void
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $mockConnectionFactory = $this->getMockBuilder(Factory::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+        ]);
+
+        $mockStream
+            ->expects($this->atLeast(3))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(6))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                1000,
+                1000,
+                1000
+            );
+
+        $mockStream
+            ->expects($this->exactly(3))
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+OK\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $streamConnection = new StreamConnection($parameters, $mockStreamFactory);
+        $connection = new RedisCluster($mockConnectionFactory, $parameters);
+        $connection->add($streamConnection);
+
+        $pipeline = new Pipeline(new Client($connection));
+
+        $responses = $pipeline->execute(function (Pipeline $pipe) {
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+        });
+
+        $this->assertEquals(['OK', 'OK', 'OK'], $responses);
+    }
+
+    /**
+     * @group disconnected
+     * @throws Exception
+     */
+    public function testExecutePipelineInvokesOnAggregateConnectionFailCallbackOnConnectionException(): void
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $mockConnectionFactory = $this->getMockBuilder(Factory::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+        ]);
+
+        $streamConnection = new StreamConnection($parameters, $mockStreamFactory);
+        $connection = $this->getMockBuilder(RedisCluster::class)
+            ->setConstructorArgs([$mockConnectionFactory, $parameters])
+            ->onlyMethods(['getConnectionByCommand', 'remove', 'askSlotMap'])
+            ->getMock();
+
+        // Disable useClusterSlots to avoid askSlotMap() calls
+        $connection->useClusterSlots = false;
+
+        $mockStream
+            ->expects($this->atLeast(3))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(6))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new \Predis\Connection\ConnectionException($streamConnection, 'Connection failed')),
+                $this->throwException(new \Predis\Connection\ConnectionException($streamConnection, 'Connection failed')),
+                $this->throwException(new \Predis\Connection\ConnectionException($streamConnection, 'Connection failed')),
+                1000,
+                1000,
+                1000
+            );
+
+        $mockStream
+            ->expects($this->exactly(3))
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+OK\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $connection
+            ->expects($this->exactly(9))
+            ->method('getConnectionByCommand')
+            ->willReturn($streamConnection);
+
+        // Verify that remove() is called on the aggregate connection during retry
+        $connection
+            ->expects($this->exactly(3))
+            ->method('remove')
+            ->with($streamConnection);
+
+        // Verify that askSlotMap() is NOT called since useClusterSlots is false
+        $connection
+            ->expects($this->never())
+            ->method('askSlotMap');
+
+        $pipeline = new Pipeline(new Client($connection));
+
+        $responses = $pipeline->execute(function (Pipeline $pipe) {
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+        });
+
+        $this->assertEquals(['OK', 'OK', 'OK'], $responses);
+    }
+
+    /**
+     * @group disconnected
+     * @throws Exception
+     */
+    public function testExecutePipelineInvokesOnAggregateConnectionFailCallbackOnTimeoutException(): void
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $mockConnectionFactory = $this->getMockBuilder(Factory::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+        ]);
+
+        $streamConnection = new StreamConnection($parameters, $mockStreamFactory);
+        $connection = $this->getMockBuilder(RedisCluster::class)
+            ->setConstructorArgs([$mockConnectionFactory, $parameters])
+            ->onlyMethods(['getConnectionByCommand', 'remove'])
+            ->getMock();
+
+        // Disable useClusterSlots to avoid askSlotMap() calls
+        $connection->useClusterSlots = false;
+
+        $mockStream
+            ->expects($this->atLeast(3))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(6))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new TimeoutException($streamConnection, 0)),
+                $this->throwException(new TimeoutException($streamConnection, 0)),
+                $this->throwException(new TimeoutException($streamConnection, 0)),
+                1000,
+                1000,
+                1000
+            );
+
+        $mockStream
+            ->expects($this->exactly(3))
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+OK\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $connection
+            ->expects($this->exactly(9))
+            ->method('getConnectionByCommand')
+            ->willReturn($streamConnection);
+
+        // Verify that remove() is NOT called for TimeoutException
+        $connection
+            ->expects($this->never())
+            ->method('remove');
+
+        $pipeline = new Pipeline(new Client($connection));
+
+        $responses = $pipeline->execute(function (Pipeline $pipe) {
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+        });
+
+        $this->assertEquals(['OK', 'OK', 'OK'], $responses);
+    }
+
+    /**
+     * @group disconnected
+     * @throws Exception
+     */
+    public function testRetryReplicationPipelineOnRetryableErrors(): void
+    {
+        $mockStream = $this->getMockBuilder(StreamInterface::class)->getMock();
+        $mockStreamFactory = $this->getMockBuilder(StreamFactoryInterface::class)->getMock();
+        $parameters = new Parameters([
+            'retry' => new Retry(new ExponentialBackoff(1000, 10000), 3),
+            'role' => 'master',
+        ]);
+
+        $mockStream
+            ->expects($this->atLeast(3))
+            ->method('close')
+            ->withAnyParameters();
+
+        $mockStream
+            ->expects($this->exactly(6))
+            ->method('write')
+            ->withAnyParameters()
+            ->willReturnOnConsecutiveCalls(
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                $this->throwException(new RuntimeException('', 2)),
+                1000,
+                1000,
+                1000
+            );
+
+        $mockStream
+            ->expects($this->exactly(3))
+            ->method('read')
+            ->withAnyParameters()
+            ->willReturn("+OK\r\n");
+
+        $mockStreamFactory
+            ->expects($this->exactly(4))
+            ->method('createStream')
+            ->withAnyParameters()
+            ->willReturn($mockStream);
+
+        $streamConnection = new StreamConnection($parameters, $mockStreamFactory);
+
+        $connection = new MasterSlaveReplication(new ReplicationStrategy());
+        $connection->add($streamConnection);
+
+        $pipeline = new Pipeline(new Client($connection));
+
+        $responses = $pipeline->execute(function (Pipeline $pipe) {
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+            $pipe->set('key', 'value');
+        });
+
+        $this->assertEquals(['OK', 'OK', 'OK'], $responses);
+    }
+
     // ******************************************************************** //
     // ---- INTEGRATION TESTS --------------------------------------------- //
     // ******************************************************************** //
@@ -648,6 +980,54 @@ class PipelineTest extends PredisTestCase
         $this->assertSameValues($expectedResults, $results);
     }
 
+    /**
+     * @group connected
+     * @group relay-incompatible
+     * @requiresRedisVersion >= 6.2.0
+     * @return void
+     */
+    public function testStandaloneRetryPipelineOnTimeoutException(): void
+    {
+        $client = $this->getClient([
+            'retry' => new Retry(new ExponentialBackoff(100, 1000), 3),
+            'read_write_timeout' => 0.1,
+        ]);
+
+        $this->expectException(TimeoutException::class);
+
+        $client->pipeline(function (Pipeline $pipe) use (&$retries) {
+            $pipe->incr('test_key');
+            $pipe->blpop('foo', 3);
+        });
+        $this->assertEquals(3, $client->get('test_key'));
+    }
+
+    /**
+     * @group connected
+     * @group cluster
+     * @group relay-incompatible
+     * @requiresRedisVersion >= 6.2.0
+     * @return void
+     */
+    public function testClusterRetryPipelineOnTimeoutException(): void
+    {
+        $retries = 0;
+        $client = $this->getClient([], [
+            'parameters' => [
+                'retry' => new Retry(new ExponentialBackoff(100, 1000), 3),
+                'read_write_timeout' => 0.1,
+            ],
+        ]);
+
+        $this->expectException(TimeoutException::class);
+
+        $client->pipeline(function (Pipeline $pipe) use (&$retries) {
+            ++$retries;
+            $pipe->blpop('foo', 3);
+        });
+        $this->assertEquals(3, $retries);
+    }
+
     /**
      * @group connected
      * @group relay-incompatible
@@ -656,7 +1036,7 @@ class PipelineTest extends PredisTestCase
     {
         $parameters = $this->getDefaultParametersArray();
 
-        $client = $this->getClient(
+        $client = new Client(
             ["tcp://{$parameters['host']}:{$parameters['port']}?role=master&database={$parameters['database']}&password={$parameters['password']}"],
             ['replication' => 'predis']
         );
